package com.tzld.piaoquan.ad.engine.service.predict.impl;

import com.alibaba.fastjson.JSONArray;
import com.tzld.piaoquan.ad.engine.commons.redis.AdOwnRedisHelper;
import com.tzld.piaoquan.ad.engine.commons.redis.AlgorithmRedisHelper;
import com.tzld.piaoquan.ad.engine.service.predict.config.RuleRedisKeyConst;
import com.tzld.piaoquan.ad.engine.service.predict.PredictModelService;
import com.tzld.piaoquan.ad.engine.service.predict.config.AbConfig;
import com.tzld.piaoquan.ad.engine.service.predict.model.containner.ThresholdModelContainer;
import com.tzld.piaoquan.ad.engine.service.predict.param.RuleParamHelper;
import com.tzld.piaoquan.ad.engine.service.predict.param.ThresholdPredictModelParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.request.RoiPredictModelRequestParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.request.ThresholdPredictModelRequestParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class PredictModelServiceImpl implements PredictModelService {

    @Value("${ad.own.experiment.tier:ab_test002}")
    private String adOwnExperimentTier;

    @Autowired
    RuleParamHelper paramHelper;

    @Autowired
    AlgorithmRedisHelper redisHelper;

    @Autowired
    AbConfig abConfig;

    public  Map<String,Object> adPredict(ThresholdPredictModelRequestParam requestParam){
        Map<String,Object> result=new HashMap<>();
        String[] abParamArr=abConfig.getAbParams(requestParam.getAbTestCode(),requestParam.getAbExpInfo());
        if(abParamArr==null){
            return result;
        }
        String abtestId=abParamArr[0];
        String abTestConfigTag=abParamArr[1];

        HashMap<String,Map<String,Object>> abConfigMap=abConfig.getAbConfigMap();
        Map<String,Object> abtestParam=abConfigMap.getOrDefault(abtestId+"-"+abTestConfigTag,null);
        if(abtestParam==null){
            return result;
        }

        // Determine the group to which mid belongs
        String groupClassKey = (String) abtestParam.get("group_class_key");
        String midGroupKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_MID_GROUP + groupClassKey + ":" + requestParam.getMid();
        String midGroup = redisHelper.getString(midGroupKeyName);
        if (midGroup == null) {
            midGroup = "mean_group";
        }
        String[] noAdMidGroupList=new String[0];
        noAdMidGroupList=((JSONArray) abtestParam.get("no_ad_mid_group_list")).toArray(noAdMidGroupList);
        boolean inNoAdGroup = false;
        for (String group : noAdMidGroupList) {
            if (group.equals(midGroup)) {
                inNoAdGroup = true;
                break;
            }
        }
        //不出广告组
        if (inNoAdGroup) {
            // User is in the no-ad group, no ad should be shown
            result.put("mid_group", midGroup);
            result.put("ad_predict", 1);
            result.put("no_ad_strategy","no_ad_mid_group_with_video");
            return result;
        }

        //设置信息
        ThresholdPredictModelParam modelParam=ThresholdPredictModelParam.builder()
                .build();
        BeanUtils.copyProperties(requestParam,modelParam);
        modelParam.setDate(new Date());
        modelParam.setAbtestId(abtestId);
        modelParam.setAbTestConfigTag(abTestConfigTag);
        modelParam.setAbtestParam(abtestParam);
        modelParam.setMidGroup(midGroup);
        Object thresholdMixFunc=abtestParam.getOrDefault("threshold_mix_func","default");
        result= ThresholdModelContainer.
                getThresholdPredictModel(thresholdMixFunc.toString())
                .predict(modelParam);
        return result;
    }

    public Map<String,Object> adRecommendPredictByRoiModel(RoiPredictModelRequestParam requestParam){
        return null;
    }



}
