package com.tzld.piaoquan.ad.engine.service.predict.model.threshold;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.tzld.piaoquan.ad.engine.commons.redis.AdOwnRedisHelper;
import com.tzld.piaoquan.ad.engine.service.predict.calculator.ThresholdPredictCalculator;
import com.tzld.piaoquan.ad.engine.service.predict.config.RuleRedisKeyConst;
import com.tzld.piaoquan.ad.engine.service.predict.param.RuleParamHelper;
import com.tzld.piaoquan.ad.engine.service.predict.param.ThresholdCalculateParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.ThresholdPredictModelParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
public class BasicThresholdPredictModel extends ThresholdPredictModel{

    @Autowired
    RuleParamHelper paramHelper;

    @Autowired
    AdOwnRedisHelper redisHelper;
    @Override
    String initName() {
        return "basic";
    }

    @Override
    public Map<String, Object> predict(ThresholdPredictModelParam modelParam) {
        Map<String, Object> result = new HashMap<>();

        String groupClassKey = (String) modelParam.getAbtestParam().get("group_class_key");
        Map<String, Set<String>> noAdGroupWithVideoMapping = (Map<String, Set<String>>) modelParam.getAbtestParam().getOrDefault("no_ad_group_with_video_mapping", new HashMap<>());

        String midGroupKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_MID_GROUP + groupClassKey + ":" + modelParam.getMid();
        String midGroup = redisHelper.get(midGroupKeyName);
        if (midGroup == null) {
            midGroup = "mean_group";
        }

        if (noAdGroupWithVideoMapping.containsKey(midGroup)) {
            Set<String> videoMappingKeyList = noAdGroupWithVideoMapping.get(midGroup);
            String noAdVideos = redisHelper.get(RuleRedisKeyConst.KEY_NAME_PREFIX_NO_AD_VIDEOS + modelParam.getAppType());
            Map<String, List<String>> noAdVideosMap = new HashMap<>();
            if (noAdVideos != null) {
                noAdVideosMap = new Gson().fromJson(noAdVideos, new TypeToken<Map<String, List<String>>>() {}.getType());
            }

            Set<String> noAdVideoList = new HashSet<>();
            for (String videoMappingKey : videoMappingKeyList) {
                noAdVideoList.addAll(noAdVideosMap.getOrDefault(videoMappingKey, new ArrayList<>()));
            }

            if (noAdVideoList.contains(modelParam.getVideoId())) {
                // Video is in the no-ad list, do not show the ad
                result.put("mid_group", midGroup);
                result.put("ad_predict", 1);
                result.put("no_ad_strategy", "no_ad_mid_group_with_video");
                return result;
            }
        }
        result = predictWithRateProcess(modelParam,midGroup);
        return result;
    }

    public Map<String, Object> predictWithRateProcess(ThresholdPredictModelParam modelParam,String midGroup){
        Double groupShareRate=paramHelper.getGroupShareRate(modelParam.getAbtestParam(),modelParam.getDate(),midGroup);
        Double videoShareRate=paramHelper.getVideoShareRate(modelParam.getAbtestParam(),modelParam.getDate(),modelParam.getVideoId());
        double threshold = paramHelper.getThreshold(
                modelParam.getAbtestId(),
                modelParam.getAbTestConfigTag(),
                modelParam.getAbTestCode(),
                midGroup,
                modelParam.getCareModelStatus(),
                modelParam.getAbtestParam()
        );
        ThresholdCalculateParam calculateParam=ThresholdCalculateParam.builder()
                .groupShareRate(groupShareRate)
                .videoShareRate(videoShareRate).build();

        Double midVideoPredictRes=ThresholdPredictCalculator.basicCalculate(calculateParam);
        int adPredict=midVideoPredictRes > threshold?2:1;
        Map<String, Object> resul=new HashMap<>();
        resul.put("mid_group", midGroup);
        resul.put("group_share_rate", groupShareRate);
        resul.put("video_share_rate", videoShareRate);
        resul.put("mid_video_predict_res", midVideoPredictRes);
        resul.put("threshold", threshold);
        resul.put("ad_predict", adPredict);
        return new HashMap<>();
    }
}
